% Select geometric cells using some criterion, for instance a gcell is selected
% because all its nodes are inside a box or on its surface.
% Examples:
% Select all cells with all nodes inside the box:
%     gcell_select(fens,gcells,struct ('box',[1 -1 2 0 4 3]))
% Select all cells with at least one node inside the box:
%     gcell_select(fens,gcells,struct ('box',[1 -1 2 0 4 3],'anynode'))
% Select all cells with given label:
%     gcell_select(fens,gcells,struct ('label', 13))
%
% The option 'inflate' may be used to increase or decrease the extent of
% the box (or the distance) to make sure some nodes which would be on the
% boundary are either excluded or included.
% 
% This function uses the node selection function fenode_select() to search
% the nodes. 
function celllist = gcell_select(fens, gcells, options)
    inside = true;
    anynode = false;
    label = [];
    if isfield(options,'anynode')
        inside = false;
        anynode = true;
    end
    if isfield(options,'label')
        label = options. label;
    end
    if ~isempty(label)
        celllist= [];
        for i=1: length (gcells)
            lbl=get(gcells(i),'label');
            if label==lbl
                celllist(end+1) =i;
            end
        end
        return;
    end
    nodelist=fenode_select(fens, options);
    celllist= [];
    for i=1: length (gcells)
        conn=get(gcells(i),'conn');
        tf = ismember(conn, nodelist);
        if inside
            if sum(tf) == length(conn)
                celllist(end+1) =i;
            end
        end
        if anynode
            if sum(tf) >= 1
                celllist(end+1) =i;
            end
        end
    end
end
